Option Explicit

Const SCL As Byte = 5
Const SDA As Byte = 6

'////////////////////////////////////////////////////
' I2C BUS FUNCTION
'////////////////////////////////////////////////////
Public Sub I2C_in_byte(ByRef I_byte As Byte) 
   DIM N as Byte, Y as Byte 
   I_byte = 0
   For N = 1 to 8 Step 1
      Call I2C_high_SCL()       ' bring clock high
      Y =  GetPin(SDA)          ' read 6
      Call I2C_low_SCL()
      I_byte = I_byte * 2 + Y   ' shift left and insert Y
   Next 
   call SendAck()	
End Sub
'///////////////////////////////////////////////////////////////////////
Public Sub I2C_out_byte(ByVal O_byte As Byte)
 
   DIM N as Byte
   For N = 1 TO 8 Step 1
      If (O_byte >= 128) then   ' most sig bit is a one
         Call I2C_high_SDA()         
      Else
         Call I2C_low_SDA()     	' set 6          
      End If

      Call I2C_high_SCL()		'Clock Pulse
      Call I2C_low_SCL()		     'Clock Pulse

      O_byte = O_byte * 2       ' shift left      
   Next 
	call ReadAck()'Reads the acknowledge from the slave device.	   
End Sub
'/////////////////////////////////////////////////////////////////////////////
Public Sub ReadAck()           ' allows slave to acknowledge  

    Call I2C_high_SDA()          
    Call I2C_high_SCL()          '
	'Call I2cReport()		'Reports the status of the acknowledge condition.
	Call delay(0.0)
   	Call I2C_low_SCL()
End Sub
'///////////////////////////////////////////////////////////////////////////////
Public Sub I2cReport()

  Dim ackack as Byte
  ackack=GetPin(SDA)
  If ackack=0 Then
    'debug.print "Acknowledge Received"
		Exit Sub
	End If 
End Sub
'///////////////////////////////////////////////////////////////////////////
Public Sub SendAck()     

   Call I2C_low_SDA()
   Call I2C_high_SCL()
   Call delay(0.0)
   Call I2C_low_SCL()
   Call I2C_high_SDA()         
End Sub
'///////////////////////////////////////////////////////////////////////////
Public Sub I2C_start()          

   Call I2C_high_SDA()
   Call I2C_high_SCL()
   Call I2C_low_SDA()
   Call I2C_low_SCL()
End Sub
'////////////////////////////////////////////////////////////////////////////
Public Sub I2C_stop()          

   Call I2C_low_SCL()
   Call I2C_low_SDA()
   Call I2C_high_SCL()
   Call I2C_high_SDA()
End Sub
'///////////////////////////////////////////////////////////////////////////
Public Sub I2C_high_SDA()

   Call PutPin(SDA, 2)      ' tristate
End Sub
'//////////////////////////////////////////////////////////////////////////
Public Sub I2C_high_SCL()       

   Call PutPin(SCL, 2)        ' tristate
End Sub
'///////////////////////////////////////////////////////////////////////////
Public Sub I2C_low_SDA()

   Call PutPin(SDA, 0)      ' hard logic zero
End Sub
'/////////////////////////////////////////////////////////////////////////////
Public Sub I2C_low_SCL()
   Call PutPin(SCL, 0)      ' hard logic zero
End Sub
'////////////////////////////////////////////////////
' END I2C BUS FUNCTION
'////////////////////////////////////////////////////


'///////////////////////////////////////////////////
'DS1621 FUNCTION
'///////////////////////////////////////////////////
Public Sub DS1621_init(ByVal address As Byte)
  call I2C_stop()
  call I2C_start()
  call I2C_out_byte(144 or (address*2))
  call I2C_out_byte(172)
  call I2C_out_byte(172)
  call I2C_start()
  call I2C_out_byte(144 or (address*2))
  call I2C_out_byte(238)
  call I2C_stop()
End Sub
/////////////////////////////////////////////////////
Public Function DS1621_readtmp(ByVal address As Byte) as single
  dim Temp as single
  dim tmp(1) as byte
  tmp(0)=0
  tmp(1)=0
  Call I2C_stop()
  Call I2C_start()
  Call I2C_out_byte(144 or (address*2))
  Call I2C_out_byte(170)
  Call I2C_start()
  Call I2C_out_byte(145 or (address*2))
  Call I2C_in_byte(tmp(0))
  If(tmp(0)>127) then
    Temp = csng((not(tmp(0))) + 1)
    Temp = Temp * (-1.0)
  Else
    Temp = csng(tmp(0))
  End if
  Call I2C_in_byte(tmp(1))
  If(tmp(1)=128) then
    Temp = Temp + 0.5
  End if
  Call Delay(0.0)
  Call I2C_stop()
  DS1621_readtmp = Temp
End Function 

'///////////////////////////////////////////////////
' END DS1621 FUNCTION
'///////////////////////////////////////////////////

'PIN 5 CLOCK
'PIN 6 DATA
Public Sub Main()
  Dim Temperatura as single	
  'Inizializzazioni
  Call DS1621_init(0)
  'Lettura
  Do	
    Temperatura = DS1621_readtmp(0)
    debug.print "Temperatura: " & cstr(Temperatura)  & " C"
    Call Delay(2.0)
  Loop
End Sub
